<?php
namespace Controleur;

use Gateway\Connection;

Class FrontControler{

    private $listAction;

    private Connection $co;

    /**
     * @throws \Exception
     */
    public function __construct($co){
        global $twig,$racine;

        $this->listAction = ['visitor' => array('accueil','search','quote','login','signin','validlogin','validsignin'),
                             'user' => array('quiz','commentary','logout','addComment','favorite','profil','addFav','supFav','changedata', 'submit', 'validsubmit', 'add','deleteAccount'),
                             'admin' => array('null')];

        $dVueEreur = [];

        $router = new \AltoRouter();

        $router->setBasePath($racine); ///~kekentin/WF/WF-Website

        $router->map('GET', '/', 'VisitorControler','accueil');

        /*
        'i'  => '[0-9]++'
        'a'  => '[0-9A-Za-z]++'
        'h'  => '[0-9A-Fa-f]++'
        '*'  => '.+?'
        '**' => '.++'
        ''   => '[^/\.]++'
        */


        // Route mapping using the router object
        // Each route is associated with an HTTP method (GET|POST), a URL path, a controller, and a method within that controller.

        //Visitor routes
        $router->map('GET|POST', '/quote/[i:idQuote]', 'VisitorControler','quote');
        $router->map('GET|POST', '/search', 'VisitorControler','search');
        $router->map('GET|POST', '/login', 'VisitorControler','login');
        $router->map('GET|POST', '/signin', 'VisitorControler','signin');
        $router->map('GET|POST', '/validlogin', 'VisitorControler','validlogin');
	    $router->map('GET|POST', '/validsignin', 'VisitorControler','validsignin');

        //User routes
        $router->map('GET|POST', '/addComment', 'UserControler','addComment');
        $router->map('GET|POST', '/quiz/[i:id]', 'UserControler','quiz');
        $router->map('GET|POST', '/favorite', 'UserControler','favorite');
        $router->map('GET|POST', '/profil', 'UserControler','profil');
        $router->map('GET|POST', '/unlog', 'UserControler','unlog');
        $router->map('GET|POST', '/addFav/[i:id]', 'UserControler','addFav');
        $router->map('GET|POST', '/supFav/[i:id]', 'UserControler','supFav');
        $router->map('GET|POST', '/changedata', 'UserControler','changedata');
        $router->map('GET|POST', '/submit', 'UserControler', 'submit');
        $router->map('GET|POST', '/validsubmit', 'UserControler', 'validsubmit');
        $router->map('GET|POST', '/add', 'UserControler', 'add');
        $router->map('GET|POST', '/deleteAccount', 'UserControler', 'deleteAccount');


        $match = $router->match();
        $action = NULL;

        if(!$match){
            $dVueEreur[] = "Requête introuvable";
            $this->vueErreur($dVueEreur);
        }
        else{
            $controller=$match['target'] ?? null;

            $action = $match['name'];
            
            //If exist
            if(!$this->ifExisteAction($action)){
                $dVueEreur[] = "Action introuvable";
                $this->vueErreur($dVueEreur);
            }

            else if(!$this->verifDroit($action)){
                global $racine;
                header("Location:". $racine ."/login");
                // $action='login';
                // $controller='VisitorControler';
            }

            $controller = '\\Controleur\\' . $controller;
            $controller = new $controller($co);
            if (is_callable(array($controller, $action))) {
                call_user_func_array(array($controller, $action),
                array($match['params'])); 
            
            }
    
        }


    }

    /**
     * Checks if a given action exists within the predefined lists of actions
     * for admin, user, or visitor roles.
     *
     * @param string $action The action to check.
     * @return bool True if the action exists in any of the lists; otherwise, false.
     */
    private function ifExisteAction(string $action):bool {
        // Check if the action exists in the 'admin' action list
        if( in_array($action , $this->listAction['admin']) ||
        // Check if the action exists in the 'user' action list
            in_array($action , $this->listAction['user']) ||
            // Check if the action exists in the 'visitor' action list
            in_array($action , $this->listAction['visitor']) ) {
                return true;
            }
        return false;
    }



    /**
     * Verifies if the current user has the rights to perform a given action 
     * based on their role (admin, user, or visitor) and the predefined lists of actions.
     *
     * @param string $action The action to verify.
     * @return bool True if the user has the rights to perform the action; otherwise, false.
     */
    private function verifDroit(string $action):bool {
        // Check if the action exists in the 'admin' action list and the user is an admin
        if( in_array($action , $this->listAction['admin']) && $_SESSION["role"] == 'admin'){
            return true;
        } 
        // Check if the action exists in the 'user' action list and the user is an admin or a user
        elseif( in_array($action , $this->listAction['user']) && ($_SESSION["role"] == 'admin' || $_SESSION["role"] == 'user') ) {
            return true;
        }
        // Check if the action exists in the 'visitor' action list and the user is an admin, user, or visitor
        elseif(in_array($action , $this->listAction['visitor']) && ($_SESSION["role"] == 'admin'|| $_SESSION["role"] == 'user'|| $_SESSION["role"] == 'visitor')) {
            return true;
        }
        return false;
    }

    /**
     * Handles the display of errors by rendering an error view and optionally displaying 
     * the first error message from the provided error array.
     *
     * @param array $dVueErreur An array of error messages to be displayed.
     * @return void
     */
    private function vueErreur(array $dVueErreur){
        global $vues;
        // Display the first error message in the array, if it exists
        echo "{$dVueErreur[0]}";
        // Include and render the error view from the `$vues` global array
        require_once $vues['erreur'];
    }

}

