<?php

namespace Model;
use Entity\CommentaryEntity;
use Gateway\CommentaryGateway;
use Gateway\Gateway;


class CommentaryModel extends Model {

    /**
     * Creates a new comment associated with a specific quote and user.
     *
     * @param string $comment The content of the comment.
     * @param string $idQuote The identifier of the quote the comment is associated with.
     * @param string $idUser The identifier of the user who is creating the comment.
     * 
     * @return bool Returns true if the comment was successfully created, otherwise false.
     */
    public function createComment(string $comment, string $idQuote, string $idUser): bool {
        // Calls the gateway's create method to insert the new comment into the data source.
        return $this->gateway->create($comment, $idUser, $idQuote);
    }

    /**
     * Retrieves all comments associated with a specific quote.
     *
     * @param int $id The unique identifier of the quote.
     * 
     * @return CommentaryEntity[] Returns an array of CommentaryEntity objects representing all the comments for the given quote.
     */
    public function getComment(int $id): array {
        // Initialize an empty array to store the comment objects.
        $com = [];

        // Fetch all comments related to the given quote ID using the gateway's findByQuote method.
        $res = $this->gateway->findByQuote($id);

        // Loop through each comment in the result and create a CommentaryEntity object for each.
        foreach ($res as $comments) {
            $com[] = new CommentaryEntity(
                $comments["id_comment"],  // Comment ID
                $comments["comment"],     // The content of the comment
                $comments["datec"],       // The date the comment was created
                $comments["username"],    // The username of the user who made the comment
                $comments["imgpath"]      // The image path associated with the user who made the comment
            );
        }

        // Return the array of CommentaryEntity objects.
        return $com;
    }

}

?>