<?php

namespace Model;
use Gateway\QuestionGateway;
use Entity\QuestionEntity;
use Gateway\Gateway;

class QuestionModel extends Model
{

    /**
     * Retrieves a question by its unique identifier.
     *
     * @param int $id_question The unique identifier of the question to retrieve.
     * 
     * @return QuestionEntity|null Returns a QuestionEntity object if found, otherwise null.
     */
    public function getQuestion(int $id_question) : ?QuestionEntity {
        // Attempt to find the question by its ID using the gateway's findById method.
        $q = $this->gateway->findById($id_question);

        // If the question is found, create and return a new QuestionEntity object with the question details.
        if ($q) {
            return new QuestionEntity(
                $q[0]['id_question'],  // Question ID
                $q[0]['texte'],         // The text/content of the question
                $q[0]['answera'],       // The first answer option
                $q[0]['answerb'],       // The second answer option
                $q[0]['answerc'],       // The third answer option
                $q[0]['answerd'],       // The fourth answer option
                $q[0]['canswer']        // The correct answer
            );
        }

        // If no question is found, return null.
        return null;
    }

    /**
     * Retrieves a random question from the database.
     *
     * @return QuestionEntity|null Returns a QuestionEntity object if a random question is found, otherwise null.
     */
    public function getRdmQuestion() : ?QuestionEntity {
        // Fetch a random question using the gateway's findRdmQuestion method.
        $q = $this->gateway->findRdmQuestion();

        // If a random question is found, create and return a new QuestionEntity object with the question details.
        if ($q) {
            return new QuestionEntity(
                $q['id_question'],  // Question ID
                $q['question'],      // The text/content of the question
                $q['answerA'],       // The first answer option
                $q['answerB'],       // The second answer option
                $q['answerC'],       // The third answer option
                $q['answerD'],       // The fourth answer option
                $q['cAnswer']        // The correct answer
            );
        }

        // If no random question is found, return null.
        return null;
    }
}

?>