<?php

namespace Model;
use Entity\quizEntity;
use Gateway\QuizGateway;
use Gateway\Gateway;


class QuizModel  extends Model{

    /**
     * Retrieves a quiz by its unique identifier.
     *
     * @param int $id_quiz The unique identifier of the quiz to retrieve.
     * 
     * @return quizEntity|null Returns a quizEntity object if found, otherwise null.
     */
    public function getQuiz(int $id_quiz): ?quizEntity{
        // Attempt to find the quiz by its ID using the gateway's findQuizById method.
        $q = $this->gateway->findQuizById($id_quiz);

        // If the quiz is found, create and return a new quizEntity object with the quiz details.
        if ($q) {
            return new quizEntity(
                $q[0]['id_quiz'],  // Quiz ID
                $q[0]['nb_quest']  // The number of questions in the quiz
            );
        }

        // If no quiz is found, return null.
        return null;
    }

    /**
     * Retrieves all quizzes from the database.
     *
     * @return quizEntity[] Returns an array of quizEntity objects representing all the quizzes.
     */
    public function getAllQuiz() : array{
        // Fetch all quizzes using the gateway's findAll method.
        $q = $this->gateway->findAll();

        // Initialize an empty array to store the quiz objects.
        $quizzes = [];

        // Loop through each quiz in the result and create a quizEntity object for each.
        foreach ($q as $quiz) {
            $quizzes[] = new quizEntity(
                $quiz['id_quiz'],       // Quiz ID
                $quiz['nb_questions']   // The number of questions in the quiz
            );
        }

        // Return the array of quizEntity objects.
        return $quizzes;
    }

}

?>