<?php
    namespace Model;
    use Entity\Quote;
    use Gateway\QuoteGateway;
    use Gateway\Gateway;

    class SearchModel extends Model
    {

        /**
         * Searches for quotes based on specified filters and search criteria.
         *
         * @param string|null $type The type of quote to search for (optional).
         * @param string|null $search The search term to filter quotes (optional).
         * @param array $filtre Additional filters to refine the search (e.g., date range, category).
         * 
         * @return Quote[] Returns an array of Quote objects that match the search criteria and filters.
         */
        public function searchQuote(?string $type, ?string $search, array $filtre): array {
            // Call the gateway's search method with the provided search criteria and filters.
            $res = $this->gateway->search($type, $search, $filtre);

            // Initialize an empty array to store the resulting Quote objects.
            $tabQ = [];

            // Loop through the search results.
            foreach ($res as $q) {
                // Truncate the quote content to 150 characters, adding '...' if it's too long.
                $q["content"] = (strlen($q["content"]) > 153) ? substr($q["content"], 0, 150) . '...' : $q["content"];

                // Create a new Quote object and add it to the tabQ array.
                $tabQ[] = new Quote(
                    $q["id_quote"],   // Quote ID
                    $q["content"],     // The content of the quote (truncated if too long)
                    $q["caracter"],    // The character associated with the quote
                    $q["imgpath"],     // The image path associated with the quote (e.g., character image)
                    $q["title"],       // The title of the quote
                    $q["dates"],       // The date when the quote was made
                    $q["likes"],       // The number of likes the quote has received
                    $q["langue"]       // The language of the quote
                );
            }

            // Return the array of Quote objects.
            return $tabQ;
        }
    }
?>
